/*
 * -----------------------------------------------------------------
 * $Revision: 1.4 $
 * $Date: 2006/03/23 01:21:37 $
 * -----------------------------------------------------------------
 * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh and
 *                Aaron Collier @ LLNL
 * -----------------------------------------------------------------
 * Copyright (c) 2002, The Regents of the University of California.
 * Produced at the Lawrence Livermore National Laboratory.
 * All rights reserved.
 * For details, see sundials/shared/LICENSE.
 * -----------------------------------------------------------------
 * This is the implementation file for a simple C-language math
 * library.
 * -----------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define WANT_OBFUSCATING_OPERATORS
#include <cln/cln.h>

#include "sundials_math.h"

#define ZERO RCONST(0.0)
#define ONE  RCONST(1.0)

using namespace cln;

/*
 * The global variables cln_precision_ determines the
 * precision of floating point numbers created with the
 * RCONST macro.  It is also used explicitly in some places
 * when converting an integer to a cl_F, with code like this:
 *     cl_float(i,cln_precision_)
 * where i is an integer.
 * The default precision is 20 digits.
 */
#define CLN_DIGITS 20
float_format_t cln_precision_ = float_format(CLN_DIGITS);

/*
 * set_realtype_precision sets the precision stored
 * in the global variable cln_precision_.  The user should
 * call this before any uses of RCONST.
 */
void set_realtype_precision(int n)
{
  cln_precision_ = float_format(n);
}


realtype RPowerI(realtype base, int exponent)
{
  int i, expt;
  realtype prod;

  prod = ONE;
  expt = abs(exponent);
  for(i = 1; i <= expt; i++) prod *= base;
  if (exponent < 0) prod = ONE/prod;
  return(prod);
}

realtype RPowerR(realtype base, realtype exponent)
{
  if (base <= ZERO) return(ZERO);

  return( cl_float(realpart(expt(base,exponent)),cln_precision_) );
/*
#if defined(SUNDIALS_USE_GENERIC_MATH)
  return((realtype) pow((double) base, (double) exponent));
#elif defined(SUNDIALS_DOUBLE_PRECISION)
  return(pow(base, exponent));
#elif defined(SUNDIALS_SINGLE_PRECISION)
  return(powf(base, exponent));
#elif defined(SUNDIALS_EXTENDED_PRECISION)
  return(powl(base, exponent));
#endif
*/
}

realtype RSqrt(realtype x)
{
  if (x <= ZERO) return(ZERO);

  return(sqrt(x));
/*
#if defined(SUNDIALS_USE_GENERIC_MATH)
  return((realtype) sqrt((double) x));
#elif defined(SUNDIALS_DOUBLE_PRECISION)
  return(sqrt(x));
#elif defined(SUNDIALS_SINGLE_PRECISION)
  return(sqrtf(x));
#elif defined(SUNDIALS_EXTENDED_PRECISION)
  return(sqrtl(x));
#endif
*/
}

realtype RAbs(realtype x)
{
  return(abs(x));
/*
#if defined(SUNDIALS_USE_GENERIC_MATH)
  return((realtype) fabs((double) x));
#elif defined(SUNDIALS_DOUBLE_PRECISION)
  return(fabs(x));
#elif defined(SUNDIALS_SINGLE_PRECISION)
  return(fabsf(x));
#elif defined(SUNDIALS_EXTENDED_PRECISION)
  return(fabsl(x));
#endif
*/
}

realtype RExp(realtype x)
{
  return(exp(x));
/*
#if defined(SUNDIALS_USE_GENERIC_MATH)
  return((realtype) exp((double) x));
#elif defined(SUNDIALS_DOUBLE_PRECISION)
  return(exp(x));
#elif defined(SUNDIALS_SINGLE_PRECISION)
  return(expf(x));
#elif defined(SUNDIALS_EXTENDED_PRECISION)
  return(expl(x));
#endif
*/
}
