/*
 *  relaxosc_clncv.cpp
 *
 *  CVODE C file for the vector field named: relaxosc
 *
 *  This file was generated by the program VFGEN, and then
 *  modified by hand to work with the CVODE version that uses CLN.
 *
 *  The functions in this file define the vector field and Jacobian
 *  for the system of differential equations:
 *
 *      x' = (-y + x - x^3/3)/epsilon
 *      y' = x - a
 *
 *
 *  -- Warren Weckesser, Dept. of Mathematics, Colgate University
 */

#include <math.h>

#define WANT_OBFUSCATING_OPERATORS
#include <cln/cln.h>

#include "sundials_types.h"
#include "sundials_dense.h"
#include "cvode.h"
#include "nvector_serial.h"
#include "cvode/cvode_dense.h"

#define ZERO  RCONST(0.0)
#define ONE   RCONST(1.0)
#define THREE RCONST(3.0)

/*
 *  The vector field.
 */

int relaxosc_vf(realtype t, N_Vector y_, N_Vector f_, void *params)
    {
    realtype x, y;
    realtype epsilon, a;
    realtype *p_;

    p_ = (realtype *) params;

    x          = NV_Ith_S(y_,0);
    y          = NV_Ith_S(y_,1);

    epsilon    = p_[0];
    a          = p_[1];

    NV_Ith_S(f_,0) = ONE/epsilon*(x-(x*x*x)/THREE-y);
    NV_Ith_S(f_,1) = x-a;

    return 0;
    }

/*
 *  The Jacobian.
 */

int relaxosc_jac(long int N_, DenseMat jac_, realtype t,
                N_Vector y_, N_Vector fy_, void *params,
                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
    {
    realtype x, y;
    realtype epsilon, a;
    realtype *p_;

    p_ = (realtype *) params;

    x          = NV_Ith_S(y_,0);
    y          = NV_Ith_S(y_,1);

    epsilon    = p_[0];
    a          = p_[1];

    DENSE_ELEM(jac_, 0, 0) = ONE/epsilon*(-(x*x)+ONE);
    DENSE_ELEM(jac_, 0, 1) = -ONE/epsilon;
    DENSE_ELEM(jac_, 1, 0) = ONE;
    DENSE_ELEM(jac_, 1, 1) = ZERO;

    return 0;
    }
