/*
 *  relaxosc_cvdemo.c
 *
 *
 *  CVODE ODE solver for the vector field named: relaxosc
 *
 *  This file was generated by the program VFGEN (Version:BlueOysterCult:3)
 *  Generated on 28-May-2006 at 22:06
 *
 *  To compile and run this program, use a Makefile something like this:
 *    SUNDIALS_DIR=<directory where SUNDIALS is installed>
 *    SUNDIALS_LIB_DIR=$(SUNDIALS_DIR)/lib
 *    SUNDIALS_INC_DIR=$(SUNDIALS_DIR)/include
 *    SUNDIALS_LIBS=-lsundials_cvode -lsundials_nvecserial -lsundials_shared
 *    LIBS = -lm
 *    relaxosc_cvdemo:  relaxosc_cvdemo.o relaxosc_cv.o
 *        gcc -o relaxosc_cvdemo relaxosc_cvdemo.o relaxosc_cv.o -L$(SUNDIALS_LIB_DIR) $(SUNDIALS_LIBS) $(LIBS)
 *    relaxosc_cvdemo.o: relaxosc_cvdemo.c
 *        gcc -I$(SUNDIALS_INC_DIR) -c relaxosc_cvdemo.c
 *    relaxosc_cv.o: relaxosc_cv.c
 *        gcc -I$(SUNDIALS_INC_DIR) -c relaxosc_cv.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "sundialstypes.h"
#include "cvode.h"
#include "cvdense.h"
#include "nvector_serial.h"
#include "dense.h"

#include "relaxosc_cv_proto.h"


int use(int argc, char *argv[], int nv, char *vname[], double y_[], int np, char *pname[], double p_[])
    {
    int i;
    printf("use: %s [options]\n",argv[0]);
    printf("options:\n");
    printf("    -h    Print this help message.\n");
    for (i = 0; i < nv; ++i)
        printf("    %s=<initial_condition>   Default value is %e\n",vname[i],y_[i]);
    for (i = 0; i < np; ++i)
        printf("    %s=<parameter_value>   Default value is %e\n",pname[i],p_[i]);
    printf("    abserr=<absolute_error_tolerance>\n");
    printf("    relerr=<relative_error_tolerance>\n");
    printf("    stoptime=<stop_time>\n");
    }


int assign(char *str[], int ns, double v[], char *a)
    {
    int i;
    char name[256];
    char *e;

    e = strchr(a,'=');
    if (e == NULL)
        return(-1);
    *e = '\0';
    strcpy(name,a);
    *e = '=';
    ++e;
    for (i = 0; i < ns; ++i)
        if (strcmp(str[i],name)==0)
            break;
    if (i == ns)
        return(-1);
    v[i] = atof(e);
    return(i);
    }


int main (int argc, char *argv[])
    {
    int i,j;
    int flag;
    const int N_ = 2;
    const int P_ = 2;
    realtype def_y_[2] = {RCONST(0.0), RCONST(0.0)};
    realtype y_[2];
    realtype def_p_[2] = {RCONST(5.0000000000000003e-02), RCONST(0.0000000000000000e+00)};
    realtype p_[2];
    realtype solver_param_[3] = {RCONST(1.0e-6),RCONST(0.0),RCONST(10.0)};
    char *varnames_[2] = {"x", "y"};
    char *parnames_[2] = {"epsilon", "a"};
    char *solver_param_names_[3] = {"abserr","relerr","stoptime"};

    for (i = 0; i < N_; ++i)
        y_[i] = def_y_[i];
    for (i = 0; i < P_; ++i)
        p_[i] = def_p_[i];
    for (i = 1; i < argc; ++i)
        {
        int j;
        if (strcmp(argv[i],"-h") == 0)
            {
            use(argc,argv,N_,varnames_,def_y_,P_,parnames_,def_p_);
            exit(0);
            }
        j = assign(varnames_,N_,y_,argv[i]);
        if (j == -1)
            {
            j = assign(parnames_,P_,p_,argv[i]);
            if (j == -1)
                {
                j = assign(solver_param_names_,3,solver_param_,argv[i]);
                if (j == -1)
                    {
                    fprintf(stderr,"unknown argument: %s\n",argv[i]);
                    use(argc,argv,N_,varnames_,def_y_,P_,parnames_,def_p_); 
                    exit(-1);
                    }
                }
            }
        }

    N_Vector y0_;
    y0_ = N_VNew_Serial(N_);
    for (i = 0; i < N_; ++i)
        NV_Ith_S(y0_,i) = y_[i];

    // For non-stiff problems:
    void *cvode_mem = CVodeCreate(CV_ADAMS,CV_FUNCTIONAL);
    // For stiff problems:
    // void *cvode_mem = CVodeCreate(CV_BDF,CV_NEWTON);

    realtype t = RCONST(0.0);
    flag = CVodeMalloc(cvode_mem,relaxosc_vf,t,y0_,CV_SS,solver_param_[1],&(solver_param_[0]));
    flag = CVodeSetFdata(cvode_mem,&(p_[0]));
    flag = CVDense(cvode_mem,N_);
    flag = CVDenseSetJacFn(cvode_mem,relaxosc_jac,&(p_[0]));

    realtype t1 = solver_param_[2];
    int nout = 201;
    realtype h = t1/(nout-1);
    // Print the initial condition
    printf("%.8e",t);
    for (j = 0; j < N_; ++j)
        printf(" %.8e",NV_Ith_S(y0_,j));
    printf("\n");
    for (i = 1; i < nout; ++i)
        {
        t = i*h;
        // Advance the solution
        flag = CVode(cvode_mem,t,y0_,&t,CV_NORMAL);
        if (flag != CV_SUCCESS)
            {
            fprintf(stderr,"flag=%d\n",flag);
            break;
            }
        // Print the solution at the current time
        printf("%.8e",t);
        for (j = 0; j < N_; ++j)
            printf(" %.8e",NV_Ith_S(y0_,j));
        printf("\n");
        }

    N_VDestroy_Serial(y0_);
    CVodeFree(cvode_mem);
    }
